/*
  K2UI  RF-Kit Power Meter - V0.4
  Bare bones code to drive I2C LCD
  A0 used for voltage measurements
*/

#include <EEPROM.h>   // Shipped with IDE
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#include <math.h>

#include "lcd_module.h"

#define PROJECT   "K2UI Power Meter        "
#define VERSION   "  SW v0.4                      "

// ADI Specs
#define  ADI5513_DBM_INT               -88.00
#define  ADI5513_VOLTS_PER_DB        0.021
#define  RFKIT_INPUT_LOSS                 9.5

// Empirical calibration adjustment
#define  CALIB_ADJ           -0.41

#define  DBM_INTERCEPT   ( ADI5513_DBM_INT+RFKIT_INPUT_LOSS+ CALIB_ADJ)
#define  DB_PER_VOLT      (1.0 / ADI5513_VOLTS_PER_DB )

void  lcd_disp(int line, char* msg ) {
    lcd.setCursor(0, line-1);     
    lcd.print( msg );
}

void setup() {

  char*  adi = "     Using the ...  ADI RF Detector Surf Board Kit            ";
  setup_lcd( 0 );
  
  lcd_disp( 1, PROJECT );
  lcd_disp( 2, VERSION );  
  delay(2500);  
    
  // Scrolling display
  for( int i=0; i<37; i+=1 ) {
      lcd_disp( 2, adi+i );
      lcd_disp( 1, PROJECT );  delay( 150 );
  }
  delay(1000);
    
  lcd_disp( 1, "                         " );
  lcd_disp( 2, "                         " ); 
  
  Serial.begin(9600);
}

double  dbm2pwr( double dbm,  char *str_pwr ) {
  char buff[32];
  double pwr ;
  
  if( dbm >= 30 ) {
      pwr = pow( 10.0 , 0.1*(dbm-30) );
      dtostrf(pwr, 4, 1, buff);
      sprintf( str_pwr, " %s Watts          ",  buff );
  } else if( dbm >= 0 ) {
       pwr = pow( 10.0 ,  0.1*(dbm) );
      dtostrf(pwr, 4, 1, buff);
      sprintf( str_pwr, " %s mWatt           ", buff );
  } else if( dbm >= -30 ) {
       pwr = pow(10.0, 0.1*(dbm+30) );
        dtostrf(pwr, 4, 1, buff);
       sprintf( str_pwr, " %s uW          ", buff );
  } else if( dbm >= -60 ) {
       pwr = pow(10.0, 0.1*(dbm+60) );
       dtostrf(pwr, 4, 1, buff);
      sprintf( str_pwr, " %s nW              ", buff );  
  } else if( dbm >= -90 ) {
       pwr = pow(10.0, 0.1*(dbm+90) );
       dtostrf(pwr, 4, 1, buff);
      sprintf( str_pwr, " %s pW             ", buff ); 
  }
   pwr = pow( 10.0 , 0.1*(dbm-30) );
    return pwr ;
} 

void loop() {
  char buff1[32],  buff2[32], str_pwr[20];
  int sensorValue = analogRead(A0);
  
  double volts =  sensorValue ;
  volts  *= (5.02125 / 1023.0);
  double dbm =  DBM_INTERCEPT + DB_PER_VOLT*volts ;
  
  dtostrf(dbm, 5, 1, str_pwr);
  sprintf( buff1, "%s dBm          ", str_pwr);
  lcd.setCursor(0, 0); lcd.print( buff1 );
  
  if( dbm < -50 ) {
      lcd_disp( 2, "Out of Range      ");
  } else if( dbm > 10 ) {
      lcd_disp( 2,  "  Ouch!          ");
  } else {
      dbm2pwr( dbm, buff2); 
      lcd_disp(2, buff2 );
 }
  
  Serial.print( buff1 );
  Serial.print(" / ");
  Serial.print( buff2);
  dtostrf(volts, 6, 3, buff1);
  
  Serial.print( sensorValue );
  Serial.println( buff1 );

  delay(1000);
  
}

